import java.io.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

import com.threed.jpct.*;
import com.threed.jpct.util.*;
import com.threed.jpct.procs.*;

class DevKit {

	// JPCT
	private FrameBuffer _buffer=null;
	private World _world=null;
	
	// single terrain and a base creature
	private FocalPoint _focalPoint = null;

	private CameraManager _cameraMgr=null;
	private InputManager _inputMgr = null;

	private Frame _frame=null;
	private Graphics _gFrame=null;
	private BufferStrategy _bufferStrategy=null;
	private GraphicsDevice _device=null;

	private int _titleBarHeight=0;
	private int _leftBorderWidth=0;
	private int _fps;
	private int _lastFps;
	private long _totalFps;

	private int _pps;
	private int _lastPps;

	private boolean _isIdle=false;


	private DevKit(String[] args) {

		Config.maxPolysVisible=100000;

		_inputMgr = new InputManager();

		for(int i=0; i<args.length; i++){
			if(args[i].equals("fullscreen")){
				_inputMgr.fullscreen=true;
				Config.glFullscreen=true;
			}
			if (args[i].equals("mipmap")){
				Config.glMipmap=true;
			}
			if(args[i].equals("trilinear")){
				Config.glTrilinear=true;
			}
			if (args[i].equals("16bit")){
				Config.glColorDepth=16;
			}
			try{
				if(args[i].startsWith("width=")){
					_inputMgr.width=Integer.parseInt(args[i].substring(6));
				}
				if(args[i].startsWith("height=")){
					_inputMgr.height=Integer.parseInt(args[i].substring(7));
				}
				if(args[i].startsWith("refresh=")){
					Config.glRefresh=Integer.parseInt(args[i].substring(8));
				}
				if(args[i].startsWith("zbuffer=")){
					Config.glZBufferDepth=Integer.parseInt(args[i].substring(8));
					if(Config.glZBufferDepth==16){
						Config.glFixedBlitting=true;
					}
				}
			} catch (Exception e) {}
		}

		_isIdle=false;
		_inputMgr.switchMode=0;
      
		_totalFps=0;
    	_fps=0;
    	_lastFps=0;

    	_world=new World();
		
    	TextureConfig tc = new TextureConfig();

		Config.fadeoutLight=false;
		_world.getLights().setOverbrightLighting(Lights.OVERBRIGHT_LIGHTING_DISABLED);
		_world.getLights().setRGBScale(Lights.RGB_SCALE_2X);  
		_world.setAmbientLight(25, 30, 30);
		_world.addLight(new SimpleVector(0, -1000, 0), 10, 10, 10);
		_world.addLight(new SimpleVector(-1000, -150, 1000), 22, 5, 4);
		_world.addLight(new SimpleVector(1000, -150, -1000), 4, 2, 22);


		Config.farPlane=50000;
		
		_focalPoint = new FocalPoint();
		_focalPoint.init(_world);

  	  	_cameraMgr = new CameraManager(_world);	  
	  	_cameraMgr.addObject3D(_focalPoint.getObject3D(FocalPoint.DEFAULT_OBJECT));

		Config.tuneForOutdoor();

		initializeFrame();

		initializeBuffer();

		_inputMgr.initializeGUI(_world, _focalPoint);
	  
		System.out.println("Starting gameloop....");

		Loader.clearCache();



      	gameLoop();
	}


	private void initializeFrame() {
		if(_inputMgr.fullscreen){
			GraphicsEnvironment env=GraphicsEnvironment.getLocalGraphicsEnvironment();
			_device=env.getDefaultScreenDevice();
			GraphicsConfiguration gc = _device.getDefaultConfiguration();
			_frame=new Frame(gc);
			_frame.setUndecorated(true);
			_frame.setIgnoreRepaint(true);
			_device.setFullScreenWindow(_frame);
			if (_device.isDisplayChangeSupported()) {
				_device.setDisplayMode(new DisplayMode(_inputMgr.width, _inputMgr.height, 32, 0));
			}
			_frame.createBufferStrategy(2);
			_bufferStrategy = _frame.getBufferStrategy();
			Graphics g = _bufferStrategy.getDrawGraphics();
			_bufferStrategy.show();
			g.dispose();
		}else{
			_frame=new Frame();
			_frame.setTitle("jPCT "+Config.getVersion());
			_frame.pack();
			Insets insets = _frame.getInsets();
			_titleBarHeight=insets.top;
			_leftBorderWidth=insets.left;
			_frame.setSize(_inputMgr.width + _leftBorderWidth + insets.right, _inputMgr.height + _titleBarHeight + insets.bottom);
			_frame.setResizable(false);
			_frame.show();
			_gFrame = _frame.getGraphics();
		}

		// do this if we are going back to software from OGL...
		if(!_inputMgr.initialized())
	  		_inputMgr.init(_frame, _cameraMgr);

		_frame.addWindowListener(new WindowEvents());
	}

	private void display(){
		if(!_inputMgr.openGL) {
			if(!_inputMgr.fullscreen){
            	_buffer.display(_gFrame, _leftBorderWidth, _titleBarHeight);
         	}else{
            	Graphics g = _bufferStrategy.getDrawGraphics();
            	g.drawImage(_buffer.getOutputBuffer(), 0, 0, null);
            	_bufferStrategy.show();
            	g.dispose();
         	}
		}else{
        	_buffer.displayGLOnly();
      	}
	}

	private void initializeBuffer(){

		try{
			World.setDefaultThread(Thread.currentThread());

			_buffer=new FrameBuffer(_inputMgr.width, _inputMgr.height, FrameBuffer.SAMPLINGMODE_NORMAL);
			_buffer.enableRenderer(IRenderer.RENDERER_SOFTWARE);
			_buffer.setBoundingBoxMode(FrameBuffer.BOUNDINGBOX_NOT_USED);

	      	_buffer.optimizeBufferAccess();

			_inputMgr.switchMode = InputManager.SWITCH_RENDERER;
			switchOptions();
		}catch(Exception e){
			System.err.println("DevKit::initializeBuffer() " + e.getMessage());
			e.printStackTrace();
		}
	}
	
	private void gameLoop() {
		
		
		Timer timer=new Timer(25);
		timer.start();

		Timer fpsTimer=new Timer(1000);
		fpsTimer.start();


		while(!_inputMgr.exit){
			if(!_isIdle){

				long ticks=timer.getElapsedTicks();
				for(int i=0; i<ticks; i++) {
					
					_cameraMgr.update(_focalPoint);
					_focalPoint.checkMoveSelected();
					_inputMgr.checkMoveLight();

				}
			
				_inputMgr.processKeyboardMouse(_buffer, _world, _focalPoint);

			
				_cameraMgr.checkModifySettings();
            
				if(_inputMgr.switchMode!=0)
					switchOptions();

				_buffer.clear(Color.BLACK);
				_world.renderScene(_buffer);

				if(!_inputMgr.wireframe)
					_world.draw(_buffer);
				else
					_world.drawWireframe(_buffer, Color.white);

				if(_inputMgr.bloom)
					_buffer.runPostProcessors();

				_inputMgr.updateGUI();

            	_buffer.update();
            	display();



            	_fps++;
            	_pps+=_world.getVisibilityList().getSize();

            	if(fpsTimer.getElapsedTicks()>0){
               		_totalFps =(_fps - _lastFps);
					_lastFps = _fps;
					_lastPps = _pps;
					_pps=0;
            	}

				Thread.yield();

			}else{
            	try{
               		Thread.sleep(500);
				}catch(InterruptedException e){}
			}
		}

		if(_inputMgr.openGL) {
			_buffer.disableRenderer(IRenderer.RENDERER_OPENGL);
		}else{
			if(_inputMgr.fullscreen) {
				_device.setFullScreenWindow(null);
			}
		}
		System.exit(0);
	}

	private void switchOptions() {
		switch(_inputMgr.switchMode) {
			case(InputManager.SWITCH_RENDERER): {
            	_isIdle=true;
            	if(_buffer.usesRenderer(IRenderer.RENDERER_OPENGL)) {
               
			   		_inputMgr.destroy();
               
			   		_buffer.disableRenderer(IRenderer.RENDERER_OPENGL);
               		_buffer.enableRenderer(IRenderer.RENDERER_SOFTWARE, IRenderer.MODE_OPENGL);
               		_inputMgr.openGL=false;

               		if(_inputMgr.fullscreen)
                  		_device.setFullScreenWindow(null);

               		_frame.hide();
               		_frame.dispose();
               		
					initializeFrame();

            	}else{
               		_frame.hide();
               		_buffer.enableRenderer(IRenderer.RENDERER_OPENGL, IRenderer.MODE_OPENGL);
               		_buffer.disableRenderer(IRenderer.RENDERER_SOFTWARE);
               
			   		// darkening, blur, strength, quality
			   		BloomGLProcessor bloom=new BloomGLProcessor();
	  		   		_buffer.addPostProcessor(bloom);

			   		_inputMgr.openGL=true;
               
			   		_inputMgr.initOGL();

            	}
				_isIdle=false;
            	break;
			}
		}
		_inputMgr.switchMode=0;
	}

	private class WindowEvents extends WindowAdapter {
		public void windowIconified(WindowEvent e) {
			_isIdle=true;
		}
      	public void windowDeiconified(WindowEvent e) {
        	_isIdle=false;
      	}
	}

	private class Timer{
		private long ticks=0;
		private long granularity=0;

		public Timer(int granularity) {
			this.granularity=granularity;
		}

		public void start() {
        	ticks=System.currentTimeMillis();
		}
		public void reset() {
			start();
		}

		public long getElapsedTicks() {
			long cur=System.currentTimeMillis();
			long l=cur-ticks;
			if(l>=granularity){
				ticks=cur-(l%granularity);
				return l/granularity;
			}
			return 0;
		}
	}

	public static void main(String[] args) {
    	new DevKit(args);	
	}
}
