import org.fenggui.render.lwjgl.EventHelper;
import org.fenggui.render.lwjgl.LWJGLBinding;
import org.lwjgl.input.Mouse;
import org.lwjgl.opengl.Display;
import org.lwjgl.opengl.DisplayMode;
import org.fenggui.*;
import org.fenggui.event.*;
import org.fenggui.composites.Window;
import org.fenggui.layout.FormData;
import org.fenggui.layout.FormLayout;
import org.fenggui.layout.RowLayout;
import org.fenggui.layout.GridLayout;
import org.fenggui.layout.StaticLayout;
import org.fenggui.ComboBox;

import java.awt.Color;
import com.threed.jpct.*;
import com.threed.jpct.util.*;
import com.threed.jpct.procs.*;
import java.util.*;


public class GUIEventHandler{

	// fengui...
	org.fenggui.Display _desk = null;
	private int lastButtonDown = -1;

	private World _world = null;

	private ObjectsLoader _ol = null;
	private WidgetBuilder _wb = null;
	private CursorManager _cursorMgr = null;

	private FocalPoint _focalPoint = null;
	private HashMap _hsSelected = null;

	public ObjectsLoader getObjectsLoader(){return _ol;}

	public ObjectAttributes getObjectAttributesByName(String sname){
		ObjectAttributes oa = null;
		try{

			Object3D obj = _world.getObjectByName(sname);
			if(obj != null)
				oa = _ol.getObjectAttributes(obj);

		}catch(Exception e){
			System.err.println("GUIEventHandler::getObjectAttributes() " + e.getMessage());
			e.printStackTrace();
		}
		return oa;
	}

	private void deleteSelectedObjects(){
		try{
			if(_hsSelected != null){
				HashMap hs = new HashMap();
				Iterator it = _hsSelected.values().iterator();
				while (it.hasNext()) {
					
					ObjectAttributes oa = (ObjectAttributes)it.next();
					if(oa.obj3D.getName().startsWith(FocalPoint.FP_PREFIX)){
						hs.put(oa.key, oa);
					}else{
						_ol.removeObject(oa, _world);
						_wb.hideLightWindow();
					}
				}
				_hsSelected.clear();
				_hsSelected = null;
				_hsSelected = hs;

				syncSelectedList("");
			}

		}catch(Exception e){
			System.err.println("GUIEventHandler::deleteSelectedObjects() " + e.getMessage());
			e.printStackTrace();

		}
	}

	private void confirmDeleteSelected(){
		try{

			if(_hsSelected.size() > 0){
				final Window w = new Window(true, false, false, true);
				_desk.addWidget(w);
				w.setX(300);
				w.setY(300);
				w.setSize(300, 100);
				w.setShrinkable(false);
				w.setTitle("Confirm Action");
				w.getContentContainer().setLayoutManager(new StaticLayout());

				Label lbl = new Label("Do you really want to delete all selected objects?");
				w.getContentContainer().addWidget(lbl);
				lbl.setX(5);
				lbl.setY(30);
				lbl.setSize(250, 10);
				//lbl.setSizeToMinSize();
			
				org.fenggui.Button bok = new org.fenggui.Button("Yes");
				bok.setX(160);
				bok.setY(5);
				bok.setSizeToMinSize();
				w.addWidget(bok);
				bok.addButtonPressedListener(
					new IButtonPressedListener(){
						public void buttonPressed(ButtonPressedEvent e){
							deleteSelectedObjects();
							try{
								w.removeAllWidgets();
								w.close();
							}catch(Exception ei){}
						}
					}
				);
				org.fenggui.Button bcancel = new org.fenggui.Button("Cancel");
				bcancel.setX(200);
				bcancel.setY(5);
				bcancel.setSizeToMinSize();
				w.addWidget(bcancel);
				bcancel.addButtonPressedListener(
					new IButtonPressedListener(){
						public void buttonPressed(ButtonPressedEvent e){
							try{
								w.removeAllWidgets();
								w.close();
							}catch(Exception ei){}
						}
					}
				);
			}

			_desk.layout();


		}catch(Exception e){
			System.err.println("GUIEventHandler::confirmDeleteSelected() " + e.getMessage());
			e.printStackTrace();
		}
	}


	public void update(){
		try{
			
			_desk.display();
			Display.update();

		}catch(Exception e){
			System.err.println("GUIEventHandler::update() " + e.getMessage());
			e.printStackTrace();
		}
	}

	public void updateMouseOverProperties(String sname, float x, float y, float z){
		try{
			
			if(sname.equals("terrain")){
				_wb.updateText(Float.toString(x), _wb.TE_TERRAIN_X);
				_wb.updateText(Float.toString(y), _wb.TE_TERRAIN_Y);
				_wb.updateText(Float.toString(z), _wb.TE_TERRAIN_Z);
			}
			
		}catch(Exception e){
			System.err.println("GUIEventHandler::updateMouseOverProperties() " + e.getMessage());
			e.printStackTrace();
		}
	}

	public void doButtonAction(int iaction){
		try{
			//System.out.println("GUIEventHandler::doButtonAction() " + iaction);

			int imode = CursorManager.MODE_NORMAL;

			switch(iaction){
				case WidgetBuilder.ACTION_LOAD_XML:{
					_wb.showDialog(_desk, DevkitConfig.DEVKIT_XML_PERSISTENCE_PATH, iaction, "Load XML Dialog", "Load");
					break;
				}
				case WidgetBuilder.ACTION_SAVE_XML:{
					_wb.showDialog(_desk, DevkitConfig.DEVKIT_XML_PERSISTENCE_PATH, iaction, "Save XML Dialog", "Save");
					break;
				}
				case WidgetBuilder.ACTION_LOAD_OBJ:{
					_wb.showDialog(_desk, DevkitConfig.MODELS_PATH, iaction, "Load Objects Dialog", "Load");
					break;
				}
				case WidgetBuilder.ACTION_DELETE_OBJ:{
					confirmDeleteSelected();
					break;
				}
				case WidgetBuilder.ACTION_LOAD_PROXY:{
					_wb.showProxyChoiceDialog(_desk);	
					break;
				}
				case WidgetBuilder.ACTION_LOAD_TERRAIN:{
					_wb.showDialog(_desk, DevkitConfig.TERRAIN_PATH, iaction, "Load Terrain Dialog", "Load");
					break;
				}
				case WidgetBuilder.ACTION_LOAD_LIGHT:{
					if(!_ol.isMarkerVisible())
						_wb.showMessageText("Must set a spawn point marker first.", _desk);
					else
						_ol.loadLight(_world);
					break;
				}
				case WidgetBuilder.ACTION_SET_OBJ_SCALE:{
					PropertyChanger.updateScale(_world, _ol, _wb); 
					break;
				}
				case WidgetBuilder.ACTION_SET_OBJ_X:{
					PropertyChanger.updateLocation(_world, WidgetBuilder.GENERAL_SETTING_X, _ol, _wb); 
					break;
				}
				case WidgetBuilder.ACTION_SET_OBJ_Y:{
					PropertyChanger.updateLocation(_world, WidgetBuilder.GENERAL_SETTING_Y, _ol, _wb); 
					break;
				}
				case WidgetBuilder.ACTION_SET_OBJ_Z:{
					PropertyChanger.updateLocation(_world, WidgetBuilder.GENERAL_SETTING_Z, _ol, _wb); 
					break;
				}
				case WidgetBuilder.ACTION_SET_OBJ_X_ROT:{
					PropertyChanger.updateRotation(_world, WidgetBuilder.GENERAL_SETTING_X_ROT, _ol, _wb); 
					break;
				}
				case WidgetBuilder.ACTION_SET_OBJ_Y_ROT:{
					PropertyChanger.updateRotation(_world, WidgetBuilder.GENERAL_SETTING_Y_ROT, _ol, _wb); 
					break;
				}
				case WidgetBuilder.ACTION_SET_OBJ_Z_ROT:{
					PropertyChanger.updateRotation(_world, WidgetBuilder.GENERAL_SETTING_Z_ROT, _ol, _wb); 
					break;
				}
				case WidgetBuilder.ACTION_SET_OBJ_TRANSPARENCY:{
					PropertyChanger.updateTransparency(_world, _ol, _wb); 
					break;
				}
				case WidgetBuilder.ACTION_SET_OBJ_CULLING:{
					PropertyChanger.updateCulling(_world, _ol, _wb); 
					break;
				}


				case WidgetBuilder.ACTION_MARKER_TOGGLE:{
					if(_cursorMgr.mode != CursorManager.MODE_PLACE)
						imode = CursorManager.MODE_PLACE;
					_cursorMgr.setMode(true, null, imode);
					break;
				}
				case WidgetBuilder.ACTION_SELECT_TOGGLE:{
					if(_cursorMgr.mode != CursorManager.MODE_SELECT)
						imode = CursorManager.MODE_SELECT;
					_cursorMgr.setMode(true, null, imode);
					break;
				}
				case WidgetBuilder.ACTION_CAMERA_TOGGLE:{
					if(_cursorMgr.mode != CursorManager.MODE_CAMERA)
						imode = CursorManager.MODE_CAMERA;
					_cursorMgr.setMode(true, null, imode);
					break;
				}

				case WidgetBuilder.ACTION_SELECT_TERRAIN:{
					if(_ol.getTerrain() != null){
						ObjectAttributes oa = _ol.getObjectAttributes(_ol.getTerrain());
						checkForceDeselect(oa);	
						_ol.forceSelectTerrain();
						oa.selected = true;
						_hsSelected.put(oa.key, oa);
						syncSelectedList(oa.key);
					}
					break;
				}

				case WidgetBuilder.ACTION_INCREASE: case WidgetBuilder.ACTION_DECREASE:{

					float fdelta = _wb.getStepping();
					if(iaction == WidgetBuilder.ACTION_DECREASE)
						fdelta *= -1;

					if(_wb.getGeneralSetting() == WidgetBuilder.GENERAL_SETTING_X)
						PropertyChanger.updateLocation(_hsSelected, new SimpleVector(fdelta, 0f, 0f));
					
					if(_wb.getGeneralSetting() == WidgetBuilder.GENERAL_SETTING_Y)
						PropertyChanger.updateLocation(_hsSelected, new SimpleVector(0f, fdelta, 0f));
					
					if(_wb.getGeneralSetting() == WidgetBuilder.GENERAL_SETTING_Z)
						PropertyChanger.updateLocation(_hsSelected, new SimpleVector(0f, 0f, fdelta));
					
					if(_wb.getGeneralSetting() == WidgetBuilder.GENERAL_SETTING_X_ROT
							|| _wb.getGeneralSetting() == WidgetBuilder.GENERAL_SETTING_Y_ROT
							|| _wb.getGeneralSetting() == WidgetBuilder.GENERAL_SETTING_Z_ROT)
						PropertyChanger.updateRotation(_hsSelected, _wb.getGeneralSetting(), fdelta);
					
					if(_wb.getGeneralSetting() == WidgetBuilder.GENERAL_SETTING_TRANSPARENCY)
						PropertyChanger.updateTransparency(_hsSelected, (int)fdelta);

					if(_wb.getGeneralSetting() == WidgetBuilder.GENERAL_SETTING_SCALE)
						PropertyChanger.updateScale(_hsSelected, fdelta);

					
					break;
				}
				
			}
			updateSelectedObject(_wb.getSelectedList().getLabel().getText());

		}catch(Exception e){
			System.err.println("GUIEventHandler::doButtonAction() " + e.getMessage());
			e.printStackTrace();
		}
	}
	public void doDialogAction(String s, int iaction){
		try{
			switch(iaction){
				case WidgetBuilder.ACTION_LOAD_OBJ:{
					if(!_ol.isMarkerVisible())
						_wb.showMessageText("Must set a spawn point marker first.", _desk);
					else
						_ol.loadObjectDefault(_world, s);
					break;
				}
				case WidgetBuilder.ACTION_LOAD_TERRAIN:{
					_ol.loadTerrain(s, _world);
					_focalPoint.resetToTerrain(_ol.getTerrain());

					break;
				}
				case WidgetBuilder.ACTION_LOAD_XML:{
					_ol.loadFile(s, _focalPoint, _world, _wb, _desk);

					break;
				}
				case WidgetBuilder.ACTION_SAVE_XML:{
					_ol.saveObjects(s, _wb, _desk, _focalPoint);
					break;
				}
			}

		}catch(Exception e){
			System.err.println("GUIEventHandler::doLoadObject() " + e.getMessage());
			e.printStackTrace();
		}
	}			
	

	public void updateSelectedObject(String sname){
		try{
			
			if(!sname.equals(WidgetBuilder.NOTHING_SELECTED) && !sname.equals("")){
				
				Object3D obj = _world.getObjectByName(sname);
				if(obj != null){
					ObjectAttributes oa = _ol.getObjectAttributes(obj);
					if(oa != null){
						_wb.updateText(oa.key, _wb.TE_OBJ_KEY);
						
						String sfile = oa.file;
						if(sfile.length() > 0){
							int iloc = sfile.lastIndexOf("/");
							if(iloc > 0 && iloc < sfile.length())
								sfile = "..." + sfile.substring(iloc, sfile.length());
						}
						_wb.updateText(sfile, _wb.TE_OBJ_FILE);
						_wb.updateText(Float.toString(oa.x), _wb.TE_OBJ_X);
						_wb.updateText(Float.toString(oa.y), _wb.TE_OBJ_Y);
						_wb.updateText(Float.toString(oa.z), _wb.TE_OBJ_Z);
						_wb.updateText(Float.toString(oa.x - oa.init_x), _wb.TE_OBJ_DELTA_X);
						_wb.updateText(Float.toString(oa.y - oa.init_y), _wb.TE_OBJ_DELTA_Y);
						_wb.updateText(Float.toString(oa.z - oa.init_z), _wb.TE_OBJ_DELTA_Z);
						_wb.updateText(Float.toString(oa.x_rot), _wb.TE_OBJ_X_ROT);
						_wb.updateText(Float.toString(oa.y_rot), _wb.TE_OBJ_Y_ROT);
						_wb.updateText(Float.toString(oa.z_rot), _wb.TE_OBJ_Z_ROT);
						_wb.updateText(Float.toString(oa.x_rot - oa.init_x_rot), _wb.TE_OBJ_DELTA_X_ROT);
						_wb.updateText(Float.toString(oa.y_rot - oa.init_y_rot), _wb.TE_OBJ_DELTA_Y_ROT);
						_wb.updateText(Float.toString(oa.z_rot - oa.init_z_rot), _wb.TE_OBJ_DELTA_Z_ROT);
						_wb.updateText(Integer.toString(oa.transparency), _wb.TE_OBJ_TRANSPARENCY);
						_wb.updateText(Float.toString(oa.scale), _wb.TE_OBJ_SCALE);
						_wb.updateText(Float.toString(oa.scale - oa.init_scale), _wb.TE_OBJ_DELTA_SCALE);

						
						String s = "false";
						if(oa.culling)
							s = "true";
						_wb.updateText(s, _wb.TE_OBJ_CULLING);						
					}
					_wb.buildExtensionsWindow(oa, _desk);
				}

			}else{
				_wb.updateText(_wb.BLANK_VALUE, _wb.TE_OBJ_KEY);
				_wb.updateText(_wb.BLANK_VALUE, _wb.TE_OBJ_FILE);
				_wb.updateText(_wb.BLANK_VALUE, _wb.TE_OBJ_X);
				_wb.updateText(_wb.BLANK_VALUE, _wb.TE_OBJ_Y);
				_wb.updateText(_wb.BLANK_VALUE, _wb.TE_OBJ_Z);
				_wb.updateText(_wb.BLANK_VALUE, _wb.TE_OBJ_DELTA_X);
				_wb.updateText(_wb.BLANK_VALUE, _wb.TE_OBJ_DELTA_Y);
				_wb.updateText(_wb.BLANK_VALUE, _wb.TE_OBJ_DELTA_Z);
				_wb.updateText(_wb.BLANK_VALUE, _wb.TE_OBJ_X_ROT);
				_wb.updateText(_wb.BLANK_VALUE, _wb.TE_OBJ_Y_ROT);
				_wb.updateText(_wb.BLANK_VALUE, _wb.TE_OBJ_Z_ROT);
				_wb.updateText(_wb.BLANK_VALUE, _wb.TE_OBJ_DELTA_X_ROT);
				_wb.updateText(_wb.BLANK_VALUE, _wb.TE_OBJ_DELTA_Y_ROT);
				_wb.updateText(_wb.BLANK_VALUE, _wb.TE_OBJ_DELTA_Z_ROT);
				_wb.updateText(_wb.BLANK_VALUE, _wb.TE_OBJ_TRANSPARENCY);
				_wb.updateText(_wb.BLANK_VALUE, _wb.TE_OBJ_SCALE);
				_wb.updateText(_wb.BLANK_VALUE, _wb.TE_OBJ_DELTA_SCALE);
				_wb.updateText(_wb.BLANK_VALUE, _wb.TE_OBJ_CULLING);

				_wb.hideExtensionsWindow();
			}

		}catch(Exception e){
			System.err.println("WidgetBuilder::updateSelectedObject() " + e.getMessage());
			e.printStackTrace();
		}
	}

	
	public void setCursorMgr(CursorManager cm){
		_cursorMgr = cm;
	}

	public void init(World w, FocalPoint fp){

		try{
			_focalPoint  = fp;

			_ol = new ObjectsLoader();

			_world = w;
			// init. the LWJGL Binding
			LWJGLBinding binding = new LWJGLBinding();
		
			_desk = new org.fenggui.Display(binding);
			// build a simple test FengGUI-Window
			//Everything everything = new Everything();
			//everything.buildGUI(_desk);

			_wb = new WidgetBuilder();
			_wb.init(this, _desk);

			_hsSelected = new HashMap();

			updateSelectedObject("");

		}catch(Exception e){
			System.err.println("GUIEventHandler::init() " + e.getMessage());
			e.printStackTrace();
		}
	}
	
	public void processKey(){
		try{
			_desk.fireKeyPressedEvent(EventHelper.mapKeyChar(), EventHelper.mapEventKey());
			_desk.fireKeyTypedEvent(EventHelper.mapKeyChar());
		}catch(Exception e){
			System.err.println("GUIEventHandler::processKey() " + e.getMessage());
			e.printStackTrace();
		}
	}

	public boolean mouseAction() {
		
	  
		boolean hitGUI = false;

		try{
		    int x = Mouse.getX();
			int y = Mouse.getY();

			// @todo the click count is not considered in LWJGL! #  
			if(lastButtonDown != -1 && Mouse.isButtonDown(lastButtonDown)) {
				hitGUI |= _desk.fireMouseDraggedEvent(x, y, 
						  EventHelper.getMouseButton(lastButtonDown));
			}else{			  
				if(Mouse.getDX() != 0 || Mouse.getDY() != 0)
					hitGUI |= _desk.fireMouseMovedEvent(x, y);
			  
				if(lastButtonDown != -1) {
					hitGUI |= _desk.fireMouseReleasedEvent(x, y, 
						EventHelper.getMouseButton(lastButtonDown), 1);
					lastButtonDown = -1;
				}
				while(Mouse.next()) {
					if(Mouse.getEventButton() != -1 && Mouse.getEventButtonState()) {
				    	lastButtonDown = Mouse.getEventButton();
				       	hitGUI |= _desk.fireMousePressedEvent(x, y, 
				        			EventHelper.getMouseButton(lastButtonDown), 1);
					}
					int wheel = Mouse.getEventDWheel();
					if(wheel != 0){
						hitGUI |= _desk.fireMouseWheel(x, y, wheel > 0, 1);
				    }
				} 
			}
		}catch(Exception e){
			System.err.println("GUIEventHandler::mouseAction() " + e.getMessage());
			e.printStackTrace();
		}

		return hitGUI;
	}


	public void checkPlaceMarker(SimpleVector v, boolean btdown, World w){
		try{
				if(_cursorMgr.mode == CursorManager.MODE_PLACE && btdown){
					_ol.placeMarker(v, w);
					
				}
		}catch(Exception e){
			System.err.println("GUIEventHandler::checkPlaceMarker() " + e.getMessage());
			e.printStackTrace();
		}
	}

	public void checkPlaceFocalPoint(SimpleVector v, boolean btdown, World w){
		try{
				if(_cursorMgr.mode == CursorManager.MODE_CAMERA && btdown){
					_focalPoint.placeFocalPoint(v);
					
				}
		}catch(Exception e){
			System.err.println("GUIEventHandler::checkPlaceFocalPoint() " + e.getMessage());
			e.printStackTrace();
		}
	}

	public void checkToggleSelect(Object3D obj, boolean btdown, CameraManager cammgr){
		try{
			
			if(!obj.getName().equals(ObjectsLoader.TERRAIN)
					&& !obj.getName().equals(ObjectsLoader.MARKER)
						&& !obj.getName().startsWith(FocalPoint.FP_PREFIX)){
				if(_hsSelected == null)
					_hsSelected = new HashMap();

				if(_cursorMgr.mode == CursorManager.MODE_SELECT){
					ObjectAttributes oa = _ol.getObjectAttributes(obj);
				
					if(oa.selected && btdown){
						obj.clearAdditionalColor();
						oa.obj3D = obj;
						oa.selected = false;
						_hsSelected.remove(oa.key);
						_ol.checkSelectedLight(oa, _wb, _desk);
						syncSelectedList("");
					}else{
						if(btdown){
							obj.setAdditionalColor(Color.WHITE);
							oa.obj3D = obj;
							oa.selected = true;
							_hsSelected.put(oa.key, oa);
							syncSelectedList(oa.key);
							_ol.checkSelectedLight(oa, _wb, _desk);
						}
					}
					
					checkForceDeselect(oa);				
				}				
			}


		}catch(Exception e){
			System.err.println("GUIEventHandler::checkToggleSelect() " + e.getMessage());
			e.printStackTrace();
		}
	}

	private void checkForceDeselect(ObjectAttributes oa){
		try{
			boolean bforce = false;

			if(oa.obj3D.getName().equals(ObjectsLoader.TERRAIN)){
				bforce = true;
				_ol.forceDeselectLights(_wb, _hsSelected);
			}else if(oa.obj3D.getName().startsWith(ObjectsLoader.LIGHT_PREFIX) && _ol.isLightSelected()){
				bforce = true;
				_ol.forceDeselectTerrain();
			}else{
				
				_ol.forceDeselectLights(_wb, _hsSelected);
				_ol.forceDeselectTerrain();
			}
			
			if(bforce){
				HashMap hs = new HashMap();
				if(_hsSelected != null){
					Iterator it = _hsSelected.values().iterator();
					while (it.hasNext()) {
						ObjectAttributes oa2 = (ObjectAttributes)it.next();
						if(!oa2.obj3D.getName().equals(oa.obj3D.getName())){
							oa2.obj3D.clearAdditionalColor();
							oa2.selected = false;
						}else{
							hs.put(oa2.key, oa2);
						}
					}
				}
				_hsSelected.clear();
				_hsSelected = null;
				_hsSelected = hs;
				syncSelectedList("");
			}

		}catch(Exception e){
			System.err.println("GUIEventHandler::checkForceDeselect() " + e.getMessage());
			e.printStackTrace();
		}
	}

	private void syncSelectedList(String smatch){
		try{

			_wb.getSelectedList().getList().clear();

			_wb.getSelectedList().getLabel().setText("");

			if(_hsSelected != null){
				Iterator it = _hsSelected.values().iterator();
				int i = -1;
				while (it.hasNext()) {
					ObjectAttributes oa = (ObjectAttributes)it.next();
					if(validEditable(oa)){
						i++;
						_wb.getSelectedList().getList().addItem(oa.key);
						if(oa.key.equals(smatch))
							_wb.getSelectedList().getList().setSelectedIndex(i, true);
					}
				}

				if(i == -1){
					_wb.getSelectedList().getList().addItem(WidgetBuilder.NOTHING_SELECTED);
					_wb.getSelectedList().getList().setSelectedIndex(0, true);
				}else{
					if(_wb.getSelectedList().getLabel().getText().equals(""))
						_wb.getSelectedList().getList().setSelectedIndex(i, true);
				}
			}

			updateSelectedObject(_wb.getSelectedList().getLabel().getText());

		}catch(Exception e){
			System.err.println("GUIEventHandler::syncSelectedList() " + e.getMessage());
			e.printStackTrace();
		}
	}

	private boolean validEditable(ObjectAttributes oa){
		boolean bOK = true;
		try{
			if(oa.obj3D.getName().startsWith(ObjectsLoader.LIGHT_PREFIX) ||
					oa.obj3D.getName().startsWith(FocalPoint.FP_PREFIX))
				bOK = false;

		}catch(Exception e){
			System.err.println("GUIEventHandler::validEditable() " + e.getMessage());
			e.printStackTrace();
		}
		return bOK;
	}

	public void createProxyObject(String s){
		boolean bOK = true;
		try{
			if(s.equals("")){
				_wb.showMessageText("Must specify a proxy type first.", _desk);
			}else{
				_ol.placeProxyObjectType(_world, s);
			}
		}catch(Exception e){
			System.err.println("GUIEventHandler::createProxyObject() " + e.getMessage());
			e.printStackTrace();
		}
	}
}