import java.io.*;
import java.util.*;
import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;
import com.threed.jpct.*;
import com.threed.jpct.util.*;
import com.threed.jpct.procs.*;
import java.awt.Color;
import org.fenggui.Display;

public class ObjectsLoader{

	

	public static final String TERRAIN = "terrain";
	public static final String LIGHT_PREFIX = "light.";
	public static final String PROXY_PREFIX = "proxy.";
	public static final String MARKER = "marker";
	public static final String PROXY_TYPE = "proxy-type";
	public static final String PROXY_TYPE_SPAWN_POINT = "spawn-point";	
	public static final String PROXY_TYPE_PORTAL = "portal";	
	public static final String PROXY_TYPE_TRIGGER = "trigger";	
	public static final String PROXY_TEXTURE_SPAWN_POINT = "PROXY_SP.PNG";	
	public static final String PROXY_TEXTURE_PORTAL = "PROXY_P.PNG";	
	public static final String PROXY_TEXTURE_TRIGGER = "PROXY_T.PNG";	

	private static final String OBJECTS_CONFIG = "objects-config";
	private static final String OBJECTS = "objects";
	private static final String OBJECT = "object";
	
	public static final String DEVKIT_OBJECT_TYPE_PROXY = "proxy";
	public static final String DEVKIT_OBJECT_TYPE_LIGHT = "light";
	public static final String DEVKIT_OBJECT_TYPE_NORMAL = "normal";

	private static final String FOCALPOINT = "focalpoint";
	private static final String FOCALPOINT_X = "focalpoint-x";
	private static final String FOCALPOINT_Y = "focalpoint-y";
	private static final String FOCALPOINT_Z = "focalpoint-z";

	private static final String DEVKIT_OBJECT_TYPE = "devkit-object-type";
	private static final String KEY = "key";
	private static final String FILE = "file";

	private static final String INIT_X = "init_x";
	private static final String INIT_Y = "init_y";
	private static final String INIT_Z = "init_z";
	private static final String INIT_X_ROT = "init_x_rot";
	private static final String INIT_Y_ROT = "init_y_rot";
	private static final String INIT_Z_ROT = "init_z_rot";	
	private static final String INIT_SCALE = "init_scale";
	private static final String CULLING = "culling";
	private static final String TRANSPARENCY = "transparency";
	private static final String ATTENUATION = "attenuation";
	private static final String INTENSITY_R = "intensity-r";
	private static final String INTENSITY_G = "intensity-g";
	private static final String INTENSITY_B = "intensity-b";
	private static final String EXTENSIONS = "extensions";


	private static final int MAX_LIGHTS = 25;

	private HashMap _hsOA = null;
	private HashMap _hsL = null;

	private String _terrain_file = "";
	private Object3D _terrain  = null;
	private Object3D _marker = null;
	private ObjectAttributes _selected_light = null;
	private boolean _terrain_selected = false;

	public boolean light_up = false;
	public boolean light_down = false;
	public boolean light_left = false;
	public boolean light_right = false;
	public boolean light_forward = false;
	public boolean light_backward = false;


	private int _light_count = -1;
	private int _proxy_count = -1;

	

	public SimpleVector getInitFocalPointVector(){return new SimpleVector();}


	public SimpleVector getObjectLoadVector(){
		SimpleVector v = null;
		try{
			if(_marker != null)
				v = new SimpleVector(_marker.getTransformedCenter());
			else
				v = new SimpleVector(0f, -5f, 0f);

		}catch(Exception e){
			System.err.println("ObjectLoader::getObjectLoadVector() " + e.getMessage());
			e.printStackTrace();
		}
		return v;
	}
	
	public Object3D getTerrain(){return _terrain;}

	public String getTerrainFile(){return _terrain_file;}
	
	public boolean isMarkerVisible(){
		boolean bret = false;
		try{

			if(_marker != null)
				bret = true;

		}catch(Exception e){
			System.err.println("ObjectsLoader::isMarkerVisible() " + e.getMessage());
			e.printStackTrace();
		}
		return bret;
	}

	public void forceDeselectTerrain(){
		try{
			if(_terrain != null){
				_terrain_selected = false;
				_terrain.clearAdditionalColor();
			}
		}catch(Exception e){
			System.err.println("ObjectsLoader::forceDeselectTerrain() " + e.getMessage());
			e.printStackTrace();
		}
	}

	public void forceSelectTerrain(){
		try{
			if(_terrain != null){
				_terrain_selected = true;
				_terrain.setAdditionalColor(Color.WHITE);
			}
		}catch(Exception e){
			System.err.println("ObjectsLoader::forceSelectTerrain() " + e.getMessage());
			e.printStackTrace();
		}
	}

	public boolean isTerrainSelected(){return _terrain_selected;}

	public boolean isLightSelected(){return _selected_light != null;}

	public void forceDeselectLights(WidgetBuilder wb, HashMap hsSelected){
		try{
			if(_selected_light != null)
				_selected_light.obj3D.clearAdditionalColor();
			_selected_light = null;

			HashMap hs = new HashMap();
			if(hsSelected != null){
				Iterator it = hsSelected.values().iterator();
				while (it.hasNext()) {
					ObjectAttributes oa = (ObjectAttributes)it.next();
					if(oa.obj3D.getName().startsWith(ObjectsLoader.LIGHT_PREFIX)){
						oa.obj3D.clearAdditionalColor();
						oa.selected = false;
					}else{
						hs.put(oa.obj3D.getName(), oa);
					}
				}
				hsSelected.clear();
				
				Iterator it2 = hs.values().iterator();
				while (it2.hasNext()){
					ObjectAttributes oa = (ObjectAttributes)it2.next();
					hsSelected.put(oa.key, oa);
				}
			}

			wb.hideLightWindow();

		}catch(Exception e){
			System.err.println("ObjectsLoader::forceDeselectLights() " + e.getMessage());
			e.printStackTrace();
		}
	}

	public ObjectAttributes getSelectedLight(){return _selected_light;}

	public void checkMoveSelectedLight(){
		try{

			if(isLightSelected()){
				float x = 0f;
				float y = 0f;
				float z = 0f;
				if(light_up)
					y = -1f;
				if(light_down)
					y = 1f;
				if(light_left)
					x = -1f;
				if(light_right)
					x = 1f;
				if(light_forward)
					z = 1f;
				if(light_backward)
					z = -1f;

				SimpleVector v = new SimpleVector(x, y, z);
				

				_selected_light.obj3D.translate(v);

				SimpleVector vl = _selected_light.obj3D.getTransformedCenter();
				_selected_light.light.setPosition(vl);

			}

		}catch(Exception e){
			System.out.println("FocalPoint::checkSelection() " + e.getMessage());
			e.printStackTrace();
		}
	}


	public void checkSelectedLight(ObjectAttributes oa, WidgetBuilder wb, Display desk){
		try{
			if(oa.obj3D.getName().startsWith(LIGHT_PREFIX)){
				
				ObjectAttributes curSel = _selected_light;
				
				forceDeselectLights(wb, new HashMap());
				
				boolean bselect = true;
				if(curSel != null){
					if(curSel.obj3D.getName().equals(oa.obj3D.getName()))
						bselect = false;
				}
				if(bselect){
					_selected_light = oa;
					oa.obj3D.setAdditionalColor(Color.WHITE);
					wb.showLightWindow(oa, desk);
				}				
			}
		}catch(Exception e){
			System.err.println("Spell::checkSelectedLight() " + e.getMessage());
			e.printStackTrace();
		}
	}

	private Light getLight(World w){
		Light lt = null;
		try{

			if(_hsL == null){
				_hsL = new HashMap();
				for(int i=0; i<MAX_LIGHTS; i++){
					Light ltinit = new Light(w);
					if(ltinit.isEnabled())
						ltinit.disable();
					_hsL.put(ltinit, ltinit);
				}
			}

			if(_hsL != null){
				Iterator itL = _hsL.values().iterator();
				while (itL.hasNext()) {
					Light ltmp = (Light)(itL.next());
					if(!ltmp.isEnabled()){
						System.out.println("Spell.getLight() returning a light...");
						lt = ltmp;
						break;
					}
				}
			}
		}catch(Exception e){
			System.err.println("Spell::getLight() " + e.getMessage());
			e.printStackTrace();
		}
		return lt;
	}

	public void removeObject(ObjectAttributes oa, World w){
		try{
			
			w.removeObject(oa.obj3D);

			_selected_light = null;

			if(oa.light != null){
				oa.light.disable();
				oa.light = null;
			}

			oa.getExtensions().clear();
			_hsOA.remove(oa.key);

		}catch(Exception e){
			System.err.println("ObjectLoader::removeObject() " + e.getMessage());
			e.printStackTrace();
		}
	}

	public void placeProxyObjectType(World w, String stype){
		try{
			_proxy_count++;
			Object3D obj = Primitives.getCube(2);
			obj.setName(PROXY_PREFIX + _proxy_count);
			obj.setTransparency(7);

			w.addObject(obj);
			obj.setLighting(Object3D.LIGHTING_ALL_ENABLED);
			
			String stex = PROXY_TEXTURE_SPAWN_POINT;
			if(stype.equals(PROXY_TYPE_TRIGGER))
				stex = PROXY_TEXTURE_TRIGGER;
			else if(stype.equals(PROXY_TYPE_PORTAL))
				stex = PROXY_TEXTURE_PORTAL;

			obj.build();
			obj.setTexture(stex);

			SimpleVector v = getObjectLoadVector();
			SimpleVector vo = new SimpleVector(v.x, v.y -5f, v.z);
			obj.setOrigin(vo);

			ObjectAttributes oa = getObjectAttributes(obj);
			oa.devkit_type = DEVKIT_OBJECT_TYPE_PROXY;
			oa.getExtensions().insertElementAt(new NVPair(PROXY_TYPE, stype), 0);
			
		}catch(Exception e){
			System.err.println("ObjectsLoader::placeProxyObjectType() " + e.getMessage());
			e.printStackTrace();
		}
	}


	public void placeMarker(SimpleVector v, World w){
		try{
			if(_marker == null){
				_marker = Primitives.getPlane(1,12);
				_marker.setName(MARKER);
				_marker.setTransparency(7);
				_marker.setTexture("object_marker.png");
				w.addObject(_marker);
				_marker.rotateX((float)Math.PI/2);
				_marker.setLighting(Object3D.LIGHTING_NO_LIGHTS);
				_marker.setAdditionalColor(Color.WHITE);
			}

			SimpleVector vo = new SimpleVector(v.x, v.y -5f, v.z);
			_marker.setOrigin(vo);
			
			
		}catch(Exception e){
			System.err.println("ObjectsLoader::placeMarker() " + e.getMessage());
			e.printStackTrace();
		}
	}

	public void loadLight(World w){
		try{
			SimpleVector vmarker = getObjectLoadVector();
			String spath = DevkitConfig.getInstance().getValue(DevkitConfig.DEVKIT_MODELS_PATH);
			Object3D[] alight =Loader.load3DS(spath + "/torch.3DS", .5f);      
			
			alight[0].build();
			_light_count++;
			alight[0].setName(LIGHT_PREFIX + _light_count);
			
			alight[0].rotateX((float)-Math.PI/2);
			alight[0].rotateZ((float)-Math.PI/4);
			alight[0].rotateMesh();
			alight[0].setRotationMatrix(new Matrix());
			
			w.addObject(alight[0]);

			SimpleVector vo = alight[0].getTransformedCenter();
			vo.scalarMul(-1);
			alight[0].translate(vo);

			alight[0].translate(vmarker);
			alight[0].translate(new SimpleVector(0f, -10f, 0f));

			ObjectAttributes oa = getObjectAttributes(alight[0]);
			oa.devkit_type = DEVKIT_OBJECT_TYPE_LIGHT;
			oa.light = getLight(w);

			setLightProperties(oa);
			
		}catch(Exception e){
			System.err.println("ObjectsLoader::loadLight() " + e.getMessage());
			e.printStackTrace();
		}
	}

	public void setLightProperties(ObjectAttributes oa){
		try{

			oa.light.setPosition(oa.obj3D.getTransformedCenter());

			oa.light.setAttenuation(oa.attenuation);
			SimpleVector v = new SimpleVector(oa.intensity_r, 
												oa.intensity_g,
												oa.intensity_b);
			oa.light.setIntensity(v);			

			oa.light.enable();
			

		}catch(Exception e){
			System.err.println("ObjectsLoader::setLightProperties() " + e.getMessage());
			e.printStackTrace();
		}
	}


	public void loadTerrain(String s, World w){
		try{

			if(_terrain != null){
				w.removeObject(_terrain);
				_terrain = null;
			}
			
			System.out.println("ObjectsLoader::loadTerrain() loading [" + s + "]");
			Object3D[] levelParts=Loader.load3DS(s, 10);      
			_terrain = new Object3D(0);
			for (int i=0; i<levelParts.length; i++) {
				Object3D part=levelParts[i];
				System.out.println("part.getName() " + part.getName());
		 
				part.setCenter(SimpleVector.ORIGIN);
				part.rotateX((float)-Math.PI/2);
				part.rotateMesh();
				part.setRotationMatrix(new Matrix());

				part.build();

				_terrain = Object3D.mergeObjects(_terrain, part);
			}

			_terrain.setName(TERRAIN);
			_terrain.createTriangleStrips(2);
			OcTree oc=new OcTree(_terrain, 100, OcTree.MODE_OPTIMIZED);
		    _terrain.setOcTree(oc);
		    oc.setCollisionUse(OcTree.COLLISION_USE);
			Config.collideOffset = 250;
		    _terrain.setCollisionMode(Object3D.COLLISION_CHECK_OTHERS);
		    _terrain.setCollisionOptimization(Object3D.COLLISION_DETECTION_OPTIMIZED);
			_terrain.enableLazyTransformations();
			_terrain.build();
		    w.addObject(_terrain);
			
			_terrain_file = s;


		}catch(Exception e){
			System.err.println("ObjectsLoader::loadTerrain() " + e.getMessage());
			e.printStackTrace();
		}
	}


	public ObjectAttributes getObjectAttributes(Object3D obj){
		ObjectAttributes oa = null;
		try{
			if(_hsOA == null)
				_hsOA = new HashMap();

			SimpleVector v = obj.getTransformedCenter();

			if(_hsOA.containsKey(obj.getName())){
				oa = (ObjectAttributes)_hsOA.get(obj.getName());
				oa.x = v.x;
				oa.y = v.y;
				oa.z = v.z;				
			}else{
				oa = new ObjectAttributes();
				oa.key = obj.getName();
				oa.obj3D = obj;
				oa.init_x = v.x;
				oa.init_y = v.y;
				oa.init_z = v.z;
				oa.init_scale = 1f;

				String sfile = DevkitConfig.getInstance().getValue(DevkitConfig.EXTENSIONS_TEMPLATE);
	            SAXBuilder builder = new SAXBuilder();
	            Document doc = builder.build(sfile);

				Element xR = doc.getRootElement();
			
				List lst = xR.getChildren();
				for(int i=0; i<lst.size(); i++){
					Element xE = (Element)lst.get(i);
					oa.getExtensions().addElement(new NVPair(xE.getTextTrim(), ""));
				}

				_hsOA.put(oa.key, oa);
			}

		}catch(Exception e){
			System.err.println("ObjectsLoader::getObjectAttributes() " + e.getMessage());
			e.printStackTrace();
		}
		return oa;
	}

	private void updateFocalPoint(Element xR, FocalPoint fp){
		try{
			SimpleVector v = new SimpleVector();
			v.x = Float.parseFloat(xR.getChild(FOCALPOINT).getChild(FOCALPOINT_X).getTextTrim());
			v.y = Float.parseFloat(xR.getChild(FOCALPOINT).getChild(FOCALPOINT_Y).getTextTrim());
			v.z = Float.parseFloat(xR.getChild(FOCALPOINT).getChild(FOCALPOINT_Z).getTextTrim());
			
			Object3D ofp = fp.getObject3D(fp.DEFAULT_OBJECT);
			SimpleVector vtc = ofp.getTransformedCenter();
			vtc.scalarMul(-1);
			ofp.translate(vtc);
			ofp.translate(v);

		}catch(Exception e){
			System.err.println("ObjectsLoader::updateFocalPoint() " + e.getMessage());
			e.printStackTrace();
		}
	}

	public void loadFile(String sfile, FocalPoint fp, World w, WidgetBuilder wb, Display desk){
		try{
			if(_hsOA != null){
				_hsOA.clear();
				_hsOA = null;
			}

			_hsOA = new HashMap();

			_light_count = -1;
			_proxy_count = -1;

			w.removeAllObjects();
			w.addObject(fp.getObject3D(fp.DEFAULT_OBJECT));

            SAXBuilder builder = new SAXBuilder();
            Document doc = builder.build(sfile);


			Element xR = doc.getRootElement();

			_terrain_file = xR.getChild(TERRAIN).getChild(FILE).getTextTrim();
			loadTerrain(_terrain_file, w);			

			updateFocalPoint(xR, fp);

			buildObjects(xR, w, sfile, wb, desk);

		}catch(Exception e){
			System.err.println("ObjectLoader::init() " + e.getMessage());
			e.printStackTrace();
		}
	}

	private void buildObjects(Element xR, World w, String sfile, WidgetBuilder wb, Display desk){
		String skey = "";
		try{

			List lst = xR.getChild(OBJECTS).getChildren();
			
			HashMap hs = new HashMap();

			for(int i=0; i<lst.size(); i++){
				Element e = (Element)lst.get(i);
				ObjectAttributes oa = new ObjectAttributes();
				System.out.println("ObjectsLoader::buildObjects() getting key [" + i + "]..");
				oa.key = e.getChild(KEY).getTextTrim();
				skey = oa.key;
				System.out.println("ObjectsLoader::buildObjects() found key [" + skey + "]");

				oa.file = e.getChild(FILE).getTextTrim();
				oa.devkit_type = e.getChild(DEVKIT_OBJECT_TYPE).getTextTrim();
				oa.x = Float.parseFloat(e.getChild(INIT_X).getTextTrim());
				oa.y = Float.parseFloat(e.getChild(INIT_Y).getTextTrim());
				oa.z = Float.parseFloat(e.getChild(INIT_Z).getTextTrim());

				oa.x_rot = Float.parseFloat(e.getChild(INIT_X_ROT).getTextTrim());
				oa.y_rot = Float.parseFloat(e.getChild(INIT_Y_ROT).getTextTrim());
				oa.z_rot = Float.parseFloat(e.getChild(INIT_Z_ROT).getTextTrim());
				
				oa.culling = false;
				if(e.getChild(CULLING).getTextTrim().equals("true"))
					oa.culling = true;
				
				oa.transparency = Integer.parseInt(e.getChild(TRANSPARENCY).getTextTrim());
				oa.scale = Float.parseFloat(e.getChild(INIT_SCALE).getTextTrim());

				List lste = e.getChild(EXTENSIONS).getChildren();

				for(int j=0; j<lste.size(); j++){
					Element ee = (Element)lste.get(j);
					oa.getExtensions().addElement(new NVPair(ee.getName(), ee.getTextTrim()));
				}
				if(oa.devkit_type.equals(DEVKIT_OBJECT_TYPE_LIGHT)){
					System.out.println("ObjectsLoader::buildObjects() loading light [" + oa.key + "]...");
					loadLight(w);
					oa.attenuation = Float.parseFloat(e.getChild(ATTENUATION).getTextTrim());
					oa.intensity_r = Float.parseFloat(e.getChild(INTENSITY_R).getTextTrim());
					oa.intensity_g = Float.parseFloat(e.getChild(INTENSITY_G).getTextTrim());
					oa.intensity_b = Float.parseFloat(e.getChild(INTENSITY_B).getTextTrim());
				}else if(oa.devkit_type.equals(DEVKIT_OBJECT_TYPE_PROXY)){
					System.out.println("ObjectsLoader::buildObjects() loading proxy [" + oa.key + "]...");
					String stype = e.getChild(PROXY_TYPE).getTextTrim();
					placeProxyObjectType(w, stype);
					oa.getExtensions().insertElementAt(new NVPair(PROXY_TYPE, stype), 0);
				}
				if(!oa.file.equals("")){
					System.out.println("ObjectsLoader::buildObjects() loading object [" + oa.key + "]...");
					loadObjectDefault(w, oa.file);
				}
				hs.put(oa, oa);

				System.out.println("ObjectsLoader::buildObjects() load for [" + oa.key + "] complete.");
			}

			System.out.println("ObjectsLoader::buildObjects() loop complete! ");

			
			PropertyChanger.reconcileObjectProperties(w, this, hs, _hsOA);			

			wb.showMessageText(" File (" + sfile + ") loaded.   ", desk);


		}catch(Exception e){
			System.err.println("ObjectLoader::buildObjects() failed on key [" + skey + "] " + e.getMessage());
			e.printStackTrace();
		}
	}

	public static String parseKeyName(String sname){
		String sret = sname;
		try{

			String s = sname;
			int iloc = s.lastIndexOf("_");
			if(iloc > 0 && (iloc + 1) < s.length()){
				s = s.substring(0, iloc);
			}
			sret = s;

		}catch(Exception e){
			System.err.println("ObjectLoader::parseKeyName() " + e.getMessage());
			e.printStackTrace();
		}
		return sret;
	}

	public boolean saveObjects(String sfile, WidgetBuilder wb, Display desk, FocalPoint fp){
		boolean b = false;
		try{

			if(_terrain != null){

	            Document doc = new Document(new Element(OBJECTS_CONFIG));
				Element xR = doc.getRootElement();
				
				Element xFP = new Element(FOCALPOINT);
				xR.addContent(xFP);
				SimpleVector vfp = fp.getObject3D(fp.DEFAULT_OBJECT).getTransformedCenter();
				xFP.addContent(new Element(FOCALPOINT_X).setText(Float.toString(vfp.x)));
				xFP.addContent(new Element(FOCALPOINT_Y).setText(Float.toString(vfp.y)));
				xFP.addContent(new Element(FOCALPOINT_Z).setText(Float.toString(vfp.z)));

				Element xT = new Element(TERRAIN);
				xR.addContent(xT);
				xT.addContent(new Element(KEY).setText(_terrain.getName()));
				xT.addContent(new Element(FILE).setText(_terrain_file));

				Element xOs = new Element(OBJECTS);
				xR.addContent(xOs);			

				if(_hsOA != null){
					Iterator it = _hsOA.values().iterator();
					while (it.hasNext()) {
						ObjectAttributes oa = (ObjectAttributes)(it.next());
						if(!oa.obj3D.getName().startsWith(FocalPoint.FP_PREFIX)){
							Element xO = new Element(OBJECT);
							xO.addContent(new Element(DEVKIT_OBJECT_TYPE).setText(oa.devkit_type));
							xO.addContent(new Element(KEY).setText(parseKeyName(oa.obj3D.getName())));
							xO.addContent(new Element(FILE).setText(oa.file));
							
							SimpleVector vtc = oa.obj3D.getTransformedCenter();

							xO.addContent(new Element(INIT_X).setText(Float.toString(vtc.x)));
							xO.addContent(new Element(INIT_Y).setText(Float.toString(vtc.y)));
							xO.addContent(new Element(INIT_Z).setText(Float.toString(vtc.z)));
							xO.addContent(new Element(INIT_X_ROT).setText(Float.toString(oa.x_rot)));
							xO.addContent(new Element(INIT_Y_ROT).setText(Float.toString(oa.y_rot)));
							xO.addContent(new Element(INIT_Z_ROT).setText(Float.toString(oa.z_rot)));
							xO.addContent(new Element(INIT_SCALE).setText(Float.toString(oa.scale)));
							xO.addContent(new Element(TRANSPARENCY).setText(Integer.toString(oa.transparency)));
							String sCulling = "false";
							if(oa.culling)
								sCulling = "true";
							xO.addContent(new Element(CULLING).setText(sCulling));

							if(oa.devkit_type.equals(DEVKIT_OBJECT_TYPE_LIGHT)){
								xO.addContent(new Element(ATTENUATION).setText(Float.toString(oa.attenuation)));
								xO.addContent(new Element(INTENSITY_R).setText(Float.toString(oa.intensity_r)));
								xO.addContent(new Element(INTENSITY_G).setText(Float.toString(oa.intensity_g)));
								xO.addContent(new Element(INTENSITY_B).setText(Float.toString(oa.intensity_b)));
							}else if(oa.devkit_type.equals(DEVKIT_OBJECT_TYPE_PROXY)){
								xO.addContent(new Element(PROXY_TYPE).setText(oa.getExtensionStringValue(PROXY_TYPE)));
							}
							Element xExt = new Element(EXTENSIONS);
							xO.addContent(xExt);
							Vector v = oa.getExtensions();
							for(int i=0; i < v.size(); i++){
								NVPair nv = (NVPair)v.elementAt(i);
								if(!nv.name.equals(PROXY_TYPE)){
									xExt.addContent(new Element(nv.name).setText(nv.value));
								}	
							}	
							xOs.addContent(xO);		
						}
						
					}
				}

				new XMLOutputter().output(doc, new FileWriter(sfile));
				wb.showMessageText(" File (" + sfile + ") written.   ", desk);

				b = true;
			}else{
				wb.showMessageText("At least a terrain is required to save.", desk);
			}


		}catch(Exception e){
			System.err.println("ObjectLoader::saveObjects() " + e.getMessage());
			e.printStackTrace();			
		}
		return b;
	}

	public void loadObjectDefault(World w, String spath){
		try{

			loadObject(w, spath, 1.0f, -1, false, getObjectLoadVector());

		}catch(Exception e){
			System.err.println("ObjectLoader::loadObjectDefault() " + e.getMessage());
			e.printStackTrace();			
		}
	}

	public void loadObject(World w, 
									String spath, 
									float fscale, 
									int transparency, 
									boolean culling,
									SimpleVector v){
		try{
      		Object3D[] parts=Loader.load3DS(spath, fscale);      

			for (int i=0; i<parts.length; i++) {
				Object3D part=parts[i];
				System.out.println("ObjectsLoader::loadObject() part.getName() " + part.getName());

				part.setCenter(SimpleVector.ORIGIN);
				part.rotateX((float)-Math.PI/2);
				part.rotateMesh();
				part.setRotationMatrix(new Matrix());

				part.build();
				
				w.addObject(part);

				part.setTransparency(transparency);
				part.setCulling(culling);
				part.createTriangleStrips(2);

				part.translate(v);
				
				ObjectAttributes oa = getObjectAttributes(part);
				
				if(i==0)
					oa.file = spath;
				
				oa.scale = fscale;
				oa.culling = culling;
				oa.transparency = transparency;

			}
			


		}catch(Exception e){
			System.err.println("ObjectsLoader::loadObject() " + e.getMessage());
			e.printStackTrace();
		}
	}


}